package adminkindboxreqvalidator

import (
	"fmt"

	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	validation "github.com/go-ozzo/ozzo-validation/v4"
)

const (
	MinKindBoxReq = 1
	MaxKindBoxReq = 100
)

type Repository interface {
	BenefactorExist(id int) (bool, error)
	KindBoxRequestExist(id int) (bool, error)
	TypeExist(id int) (bool, error)
	KindBoxBelongToBenefactor(benefactorID uint, kindboxID uint) (bool, error)
	PendingStatus(id uint) (bool, error)
}

type Validator struct {
	repo Repository
}

func New(repo Repository) Validator {
	return Validator{repo: repo}
}

func (v Validator) doesBenefactorExist(value interface{}) error {
	benefactorID := value.(int)
	_, err := v.repo.BenefactorExist(benefactorID)
	if err != nil {
		return fmt.Errorf(errmsg.ErrorMsgNotFound)
	}

	return nil
}

func (v Validator) doesKindBoxBelongToBenefactor(benefactorID uint) validation.RuleFunc {
	return func(value interface{}) error {
		kbId := value.(uint)
		_, err := v.repo.KindBoxBelongToBenefactor(benefactorID, kbId)
		if err != nil {
			return fmt.Errorf(errmsg.ErrorMsgNotFound)
		}

		return nil
	}
}

func (v Validator) doesKindBoxRequestExist(value interface{}) error {
	kindboxreqID := value.(int)
	_, err := v.repo.KindBoxRequestExist(kindboxreqID)
	if err != nil {
		return fmt.Errorf(errmsg.ErrorMsgNotFound)
	}

	return nil
}

func (v Validator) doesTypeExist(value interface{}) error {
	typeID := value.(int)
	_, err := v.repo.TypeExist(typeID)
	if err != nil {
		return fmt.Errorf(errmsg.ErrorMsgNotFound)
	}

	return nil
}

func (v Validator) hasPendingStatus(value interface{}) error {
	kindboxID := value.(uint)
	_, err := v.repo.PendingStatus(kindboxID)
	if err != nil {
		return fmt.Errorf(errmsg.ErrorMsgNotFound)
	}

	return nil
}
