package userkindboxvalidator

import (
	"fmt"

	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	validation "github.com/go-ozzo/ozzo-validation/v4"
)

type Repository interface {
	BenefactorExist(id uint) (bool, error)
	KindBoxExist(id uint) (bool, error)
	KindBoxBelongToBenefactor(bId uint, kbId uint) (bool, error)
}

type Validator struct {
	repo Repository
}

func New(repo Repository) Validator {
	return Validator{repo: repo}
}

func (v Validator) doesBenefactorExist(value interface{}) error {
	benefactorID := value.(uint)
	_, err := v.repo.BenefactorExist(benefactorID)
	if err != nil {
		return fmt.Errorf(errmsg.ErrorMsgNotFound)
	}

	return nil
}

func (v Validator) doesKindBoxExist(value interface{}) error {
	kindBoxID := value.(uint)
	_, err := v.repo.KindBoxExist(kindBoxID)
	if err != nil {
		return fmt.Errorf(errmsg.ErrorMsgNotFound)
	}

	return nil
}

func (v Validator) doesKindBoxBelongToBenefactor(benefactorID uint) validation.RuleFunc {
	return func(value interface{}) error {
		kbId := value.(uint)
		_, err := v.repo.KindBoxBelongToBenefactor(benefactorID, kbId)
		if err != nil {
			return fmt.Errorf(errmsg.ErrorMsgNotFound)
		}

		return nil
	}
}
